@@ -12,7 +12,7 @@ create table checked_range
   last_date             timestamptz not null
 );
 
-comment on table checked_range is 'Each row represents a range of objects in a datastore that were fetched, checksumed, and checked at some point in the past.';
+comment on table checked_range is 'Each row represents a range of objects in a datastore that were fetched, checksummed, and checked at some point in the past.';
 comment on column checked_range.range_start is 'First SWHID of the range that was checked (inclusive, possibly non-existent).';
 comment on column checked_range.range_end is 'Last SWHID of the range that was checked (inclusive, possiby non-existent).';
 comment on column checked_range.last_date is 'Date the last scrub of that range *started*.';
